﻿using System;

using DynamicWebService;
using NAVApplication;
using WindowsOS;

using System.Windows.Forms;

namespace NAVApplicationWrapper
{
    /*
     * This class represents the machine the NAV RoleTailored Client is running on.
     */
    public class ClientMachine
    {
        /*
         * The NAV Executables installed on the computer.
         */
        private NAVExecutables _executables;
        public NAVExecutables executables { get { return _executables; } set { _executables = value; } }

        /*
         * The NAV User Configuration File that is loaded when the RoleTailored Client starts.
         */
        private ClientUserConfig _NAVUserConfigFile;
        public ClientUserConfig NAVUserConfigFile { get { return _NAVUserConfigFile; } set { _NAVUserConfigFile = value; } }

        /*
         * Default Constructor to instantiate variables and determine the computer's operating system. The OS determines the
         * location of the ClientUserSettings.config file.
         */
        public ClientMachine()
        {
            executables = new NAVExecutables();

            try
            {
                WindowsVersion currentOS = GetCurrentOS();
                switch (currentOS)
                {
                    case WindowsVersion.WindowsXP:
                    case WindowsVersion.WindowsVista:
                    case WindowsVersion.Windows7:
                        NAVUserConfigFile = new ClientUserConfig(currentOS);
                        break;
                    case WindowsVersion.Unknown:
                        throw new Exception("Unable to determine your operating system!" +
                            Environment.NewLine +
                            "Unable to retrieve your ClientUserSettings.config file!");
                    default:
                        throw new Exception("You are running an unsupported operating system!");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + Environment.NewLine + ex.StackTrace,
                    "NAV Application Wrapper", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /*
         * Function GetCurrentOS
         * @return WindowsVersion A custom enumerated type found in the WindowsOS namespace.
         * 
         * Determines the operating system of the computer.
         */
        private WindowsVersion GetCurrentOS()
        {
            try
            {
                switch (Environment.OSVersion.Platform)
                {
                    case PlatformID.Win32NT:
                        switch (Environment.OSVersion.Version.Major)
                        {
                            case 0:
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                                return WindowsVersion.WindowsNT;
                            case 5:
                                if (Environment.OSVersion.Version.Minor == 0)
                                    return WindowsVersion.Windows2000;
                                else
                                    return WindowsVersion.WindowsXP;
                            case 6:
                                if (Environment.OSVersion.Version.Minor == 0)
                                    return WindowsVersion.WindowsVista;
                                else
                                    return WindowsVersion.Windows7;
                        }
                        break;                        

                    case PlatformID.Win32Windows:
                        switch(Environment.OSVersion.Version.Major)
                        {
                            case 4:
                                if (Environment.OSVersion.Version.Minor == 0)
                                    return WindowsVersion.Windows95;
                                else if (Environment.OSVersion.Version.Minor < 90)
                                    return WindowsVersion.Windows98;
                                else
                                    return WindowsVersion.WindowsME;
                        }
                        break;
                }

                return WindowsVersion.Unknown;
            }
            catch (InvalidOperationException)
            {
                return WindowsVersion.Unknown;
            }
        }

        /*
         * Method FindBestService
         * 
         * Connects to a NAV Web Service to determine the NAV Service Tier service with the fewest connections.
         */
        public void FindBestService()
        {
            try
            {
                string wsurl = NAVUserConfigFile.GetValue("Load Balancer WS");
                DynamicWebService.DynamicWebService invoker = new DynamicWebService.DynamicWebService(new Uri(wsurl));

                string[] results = invoker.InvokeMethod<string>("NAV_RTC_Load_Balancer_WS", "GetBestService",
                    new string[] { Environment.UserDomainName + @"\" + Environment.UserName }).Split(';');

                string server = results[0];
                string service = results[1];
                string port = results[2];
                NAVUserConfigFile.OverwriteConfigFile(server, service, port);

                invoker.InvokeMethod<string>("NAV_RTC_Load_Balancer_WS", "AddSession",
                    new string[] { Environment.UserName, server, service });
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message + Environment.NewLine + 
                    Environment.NewLine + ex.StackTrace + Environment.NewLine +
                    Environment.NewLine + "Inner Exception" + Environment.NewLine +
                    Environment.NewLine + ex.InnerException.Message + Environment.NewLine +
                    Environment.NewLine + ex.InnerException.StackTrace,
                    "NAV Application", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }


        /*
         * Method LaunchNAV
         * 
         * Starts the NAV RoleTailored Client executables
         */
        public void LaunchNAV()
        {
            NAVExecutables.Run();
        }
    }
}
