﻿using System;
using System.Collections.Generic;

using System.IO;
using System.Xml;
using System.Xml.XPath;

namespace NAVRTCLoadBalancer
{
    /*
     * This class represents the LoadBalancer.config file that the Microsoft Dynamics NAV RoleTailored Client uses when
     * it loads.
     */
    public class LoadBalancerConfig
    {
        /*
         * The full path and name of the LoadBalancer.config file.
         *    
         * If the file does not exist when assigning the value a FileNotFoundException is thrown.
         */
        private string _filename;
        public string filename
        {
            get
            {
                return _filename;
            }
            set
            {
                if (File.Exists(value))
                {
                    _filename = value;
                }
                else
                {
                    throw new FileNotFoundException(value + " could not be found!");
                }
            }
        }

        /*
         * The original contents of the LoadBalancer.config file. This content is stored because we do not want to read
         * from the file every time the service event fires.
         */
        private XmlDocument _originalFileContents;
        public XmlDocument originalFileContents { get { return _originalFileContents; } set { _originalFileContents = value; } }

        /*
         * Represents the key / value pairs from the LoadBalancer.config file.
         * 
         * Same key / value pair form file:
         * <!-- The timer interval between invocations of a callback in milliseconds. Recommended 60000 (60 seconds) -->
	     * <add key="Period" value="60000"></add>
         * 
         * Use the GetValue method below to retrieve the value for one of the keys in the file.
         */
        private Dictionary<string, string> _keyValuePairs;

        /*
         * Constructor
         * @param windowsVersion A custom enumeratio from the WindowsOS namespace that specified the Windows Version
         */
        public LoadBalancerConfig()
        {
            _keyValuePairs = new Dictionary<string, string>();

            filename = @"C:\Program Files (x86)\Microsoft Dynamics NAV\60\Load Balancer\LoadBalancer.config";
            LoadFile();
        }

        /*
         * Method LoadFile
         * 
         * Loads the original contents of the LoadBalancer.config file
         */
        public void LoadFile()
        {
            if (File.Exists(filename))
            {
                originalFileContents = new XmlDocument();
                originalFileContents.Load(filename);
            }
            else
            {
                throw new FileNotFoundException(filename + " could not be found!");
            }

            XPathNavigator navigator = originalFileContents.CreateNavigator();
            XPathExpression expr = navigator.Compile("/appSettings/add");
            XPathNodeIterator iterator = navigator.Select(expr);

            string key = string.Empty;
            string value = string.Empty;

            while (iterator.MoveNext())
            {
                key = iterator.Current.Clone().GetAttribute("key", "");
                value = iterator.Current.Clone().GetAttribute("value", "");
                if (!_keyValuePairs.ContainsKey(key))
                {
                    _keyValuePairs.Add(key, value);
                }
            }
        }

        /*
         * Function GetValue
         * @param key Represents the key found in the LoadBalancer.config file.
         * @return string The value of the key / value pair from the LoadBalancer.config file
         * 
         * This function will return the value associated with the key passed in. If no key is found null is returned.
         */
        public string GetValue(string key)
        {
            string value;
            if (_keyValuePairs.TryGetValue(key, out value))
            {
                return value;
            }
            else
            {
                return null;
            }
        }
    }
}
