﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;

using System.Threading;

using NAVServiceTier;

namespace NAVRTCLoadBalancer
{
    public partial class LoadBalancerService : ServiceBase
    {
        /*
         * The timer that will fire off the service every N seconds.
         */
        private Timer ServiceTimer;

        /*
         * The time that the service actually started working.
         */
        private DateTime _workStartTime;

        /*
         * A list of the NAV Servers this service will monitor.
         */
        private List<NAVServer> NAVServerList;

        /*
         * The full path and filename of the LoadBalancer.config file. This file specifies the services startup
         * options, similar to the CustomSettings.config file for the NAV Service Tier services.
         */
        private LoadBalancerConfig _configFile;
        public LoadBalancerConfig ConfigFile { get { return _configFile; } set { _configFile = value; } }

        /*
         * Whether or not to record events in the Windows Event Log.
         */
        private bool _useEventLog;

        /*
         * Default Constructor to initialize variables.
         */
        public LoadBalancerService()
        {
            _configFile = new LoadBalancerConfig();
            _useEventLog = (_configFile.GetValue("LogEvents").ToUpper().Equals("TRUE"));
            
            InitializeComponent();
            ServiceName = "Microsoft Dynamics NAV RTC Load Balancer";

            if(_useEventLog)
            {
                if (!EventLog.SourceExists("Microsoft Dynamics NAV RTC Load Balancer"))
                {
                    EventLog.CreateEventSource(
                       "Microsoft Dynamics NAV RTC Load Balancer", "Microsoft Dynamics NAV RTC Load Balancer Log");
                }
            }             
        }

        /*
         * Method OnStart
         * @param args The arguments that specify how this service should start.
         * 
         * Fires when the service is initially started.
         */
        protected override void OnStart(string[] args)
        {
            TimerCallback timerDelegate = new TimerCallback(DoWork);
            long delay = Int32.Parse(_configFile.GetValue("Delay"));
            long period = Int32.Parse(_configFile.GetValue("Period"));
            ServiceTimer = new Timer(timerDelegate, null, delay, period);

            LoadNAVServices();
        }

        /*
         * Method OnStop
         * 
         * Fires when the service is stopped.
         */
        protected override void OnStop()
        {
        }

        /*
         * Method LoadNAVServices
         * 
         * Loads all NAV Service Tier services found on the servers specified in the config file. Uses a NAV Web Service
         * to update a NAV table with the services that it finds.
         */
        private void LoadNAVServices()
        {
            string wsurl = _configFile.GetValue("WebService");
            DynamicWebService.DynamicWebService invoker = new DynamicWebService.DynamicWebService(new Uri(wsurl));

            NAVServerList = null;
            NAVServerList = new List<NAVServer>();            
            //string[] serverList = _configFile.GetValue("ServerList");//.Split(',');
            string serverlist = _configFile.GetValue("ServerList");
            //foreach (string server in serverList)
            //{
                NAVServerList.Add(new NAVServer(serverlist));
            //}
            NAVService service;
            foreach (NAVServer server in NAVServerList)
            {
                server.LoadNAVServices();
                foreach (KeyValuePair<string, NAVService> pair in server.NAVServices)
                {
                    service = pair.Value;
                    if (service.customSettings != null)
                    {
                        if (service.customSettings.GetValue("DatabaseName").Equals(ConfigFile.GetValue("DatabaseName")))
                        {
                            invoker.InvokeMethod<string>("NAV_RTC_Load_Balancer_WS", "AddService",
                                new string[] { server.serverName, service.customSettings.GetValue("ServerInstance"), service.customSettings.GetValue("ServerPort") });
                            invoker.InvokeMethod<string>("NAV_RTC_Load_Balancer_WS", "SetServiceStatus",
                                new string[] { server.serverName, service.customSettings.GetValue("ServerInstance"), service.customSettings.GetValue("ServerPort"), service.Status.ToString() });
                        }
                    }
                }
            }
        }

        /*
         * Method DoWork
         * @param state The current state of the service
         * 
         * Performs the actual tasks that we want our service to execute, specifically updating the NAV table with the services and
         * their statuses.
         */
        private void DoWork(object state)
        {
            if (_workStartTime != DateTime.MinValue)
            {
                // Check how much time has elapsed since work started and log any warning
                if (_useEventLog)
                {
                    EventLog.WriteEntry(
                                "Microsoft Dynamics NAV RTC Load Balancer",
                                "The load balancer has not checked on the status of any services for a while.",
                                EventLogEntryType.Warning,
                                004);
                }                
            }
            else
            {
                _workStartTime = DateTime.Now;

                try
                {
                    LoadNAVServices();
                    if(_useEventLog)
                    {
                        EventLog.WriteEntry(
                            "Microsoft Dynamics NAV RTC Load Balancer",
                            "Status Check Complete.",
                            EventLogEntryType.Information,
                            001);
                    }                    
                }
                catch (Exception e)
                {
                    if (_useEventLog)
                    {
                        EventLog.WriteEntry(
                                "Microsoft Dynamics NAV RTC Load Balancer",
                                "Unable to check status!" + Environment.NewLine + e.Message,
                                EventLogEntryType.Error,
                                003);
                    }
                }
                
                _workStartTime = DateTime.MinValue;
            }
        }
    }
}
