﻿using System.Collections.Generic;
using System.IO;
using System.Xml;
using System.Xml.XPath;

namespace NAVServiceTier
{
    /*
     * This class represents the CustomSettings.config file that the Microsoft Dynamics NAV 2009 Server uses
     * when the service starts.
     */
    public class CustomSettings
    {
        /*
         * The full path and name of the CustomSettings.config file.
         * The default location of this file is 
         *   C:\Program Files (x86)\Microsoft Dynamics NAV\60\<service instance>\CustomerSettings.config
         *   
         * If the file does not exist when assigning the value a FileNotFoundException is thrown.
         */
        private string _filename;
        public string filename 
        { 
            get 
            { 
                return _filename; 
            } 
            set 
            {
                if (File.Exists(value))
                {
                    _filename = value;
                }
                else
                {
                    throw new FileNotFoundException(value + " could not be found!");
                }
            } 
        }

        /*
         * Represents the key / value pairs from the CustomSettings.config file.
         * 
         * Same key / value pair form file:
         * <!-- Name of the Microsoft Dynamics NAV Server instance to connect to (for client) or listen on (for server). -->
         * <add key="ServerInstance" value="CanvasNAV02"></add>
         * 
         * Use the GetValue method below to retrieve the value for one of the keys in the file.
         */
        private Dictionary<string, string> _keyValuePairs;

        /*
         * Default constructor to instantiate variables.
         */
        public CustomSettings()
        {
            _keyValuePairs = new Dictionary<string, string>();
        }

        /*
         * Constructor chained to the default constructor above.
         * @param newFilename The full path and filename to the CustomSettings.config file for this NAV service.
         * 
         * This constructor is the one that should be used most often. It will automatically load the contents of the
         * CustomSettings.config file.
         */
        public CustomSettings(string newFilename) : this()
        {
            filename = newFilename;
            ReadFile();
        }

        /*
         * Function GetValue
         * @param key Represents the key found in the CustomSettings.config file.
         * @return string The value of the key / value pair from the CustomSettings.config file
         * 
         * This function will return the value associated with the key passed in. If no key is found an empty string is
         * returned.
         */
        public string GetValue(string key)
        {
            string value;
            if (_keyValuePairs.TryGetValue(key, out value))
            {
                return value;
            }
            else
            {
                return string.Empty;
            }
        }

        /*
         * Method ReadFile
         * 
         * This method loads and parses the file using XPath.
         * XPath tutorial: http://www.w3schools.com/xpath/default.asp
         * 
         * If the file does not exist a FileNotFound exception is thrown.
         */
        private void ReadFile()
        {
            if (!File.Exists(filename))
            {
                throw new FileNotFoundException(filename + " could not be found!");
            }

            XmlDocument fileContents = new XmlDocument();
            fileContents.Load(filename);

            XPathNavigator navigator = fileContents.CreateNavigator();
            XPathExpression expr = navigator.Compile("/appSettings/add");
            XPathNodeIterator iterator = navigator.Select(expr);
            string key = string.Empty;
            string value = string.Empty;

            while (iterator.MoveNext())
            {
                key = iterator.Current.Clone().GetAttribute("key", "");
                value = iterator.Current.Clone().GetAttribute("value", "");
                if (!_keyValuePairs.ContainsKey(key))
                {
                    _keyValuePairs.Add(key, value);
                }
            }
        }
    }
}
