﻿using System.ServiceProcess;
using System.IO;
using Microsoft.Win32;

using RegistryHelper;

namespace NAVServiceTier
{
    /*
     * This class represents a NAV Service Tier service. It inherits the functionality of the ServiceController class,
     * meaning that we can control it from code just like any other Windows Service, but have our own extra functionality.
     */     
    public class NAVService : ServiceController
    {
        /*
         * The path to the actual NAV Service Tier service executable, Microsoft.Dynamics.NAV.Server.exe.
         * Usually C:\Program Files (x86)\Microsoft Dynamics NAV\60\<Service Tier>
         * 
         * Retrieved from the registry using the FindPathToExecutable function.
         */
        private string _pathToExecutable;
        //public string pathToExecutable { get { return _pathToExecutable; } set { _pathToExecutable = value; } }

        /*
         * Represents the CustomSettings.config file loaded by the NAV Service Tier service when it starts.
         * 
         * Instantiated using the LoadConfiguration method.
         */
        private CustomSettings _customSettings;
        public CustomSettings customSettings { get { return _customSettings; } set { _customSettings = value; } }

        /*
         * Default constructor chained to the inherited default constructor.
         */
        public NAVService()
            : base()
        {
        }

        /*
         * Constructor chained to the inherited constructor(string, string).
         */
        public NAVService(string name, string machine)
            : base(name, machine)
        {            
        }

        /*
         * Function IsNAVService
         * @return bool Whether or not this is a NAV Service Tier service or another Windows Service
         * 
         * If the ServiceName starts with MicrosoftDynamicsNavServer then it is considered a NAV Service Tier service.
         * This will exclude the NAV Web Services that may also be running on the same server.
         */
        public bool IsNAVService()
        {
            return ServiceName.StartsWith("MicrosoftDynamicsNavServer");
        }

        /*
         * Function IsAcceptingConnections
         * @return bool Whether or not the service is accepting connections
         * 
         * A NAV Service Tier Service is considered to be accepting connections if its status is running.
         */
        public bool IsAcceptingConnections()
        {
            return Status.ToString().Equals("Running");
        }

        /*
         * Function FindPathToExecutable
         * @return string The path to the Microsoft.Dynamics.Nav.Server.exe program for this NAV Service Tier service
         * 
         * Performs a registry query to retrieve the value.
         * HKEY_LOCAL_MACHINE\System\ControlSet001\Services\<Service Name>\ImagePath
         */
        public string FindPathToExecutable()
        {
            string registrypath = @"System\ControlSet001\Services\" + this.ServiceName;
            string retval = RegistryQuery.GetRemoteKeyValue(this.MachineName, RegistryHive.LocalMachine, registrypath, "ImagePath");

            retval = retval.Replace("\"", string.Empty); //Remove any double quotes that are returned
            retval = Path.GetDirectoryName(retval);
            return retval;
        }

        /*
         * Method LoadConfiguration
         * 
         * Creates a path to the CustomSettings.config file on the remote server
         *   \\ServerName\Drive$\Path\CustomSettings.config
         * and instantiates the CustomSettings object.
         */
        public void LoadConfiguration()
        {
            _pathToExecutable = FindPathToExecutable();            
            string filename = @"\\" + this.MachineName + @"\" + _pathToExecutable + @"\CustomSettings.config";
            filename = filename.Replace(@":\", @"$\");

            if (!File.Exists(filename))
            {
                customSettings = null;
            }
            else
            {
                customSettings = new CustomSettings(filename);
            }
        }
    }
}
